@@ -28,7 +28,7 @@
 #
 ###############################################################################
 
-# $Id: Card.pm,v 1.12 2003/05/25 20:29:48 rousseau Exp $
+# $Id: Card.pm,v 1.1.1.5 2004/05/30 20:35:54 rousseau Exp $
 
 package Chipcard::PCSC::Card;
 
@@ -50,7 +50,7 @@ our $VERSION = '0.02';
 #
 # default values:
 #  $share_mode = $Chipcard::PCSC::SCARD_SHARE_EXCLUSIVE
-#  $prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0
+#  $prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0 | $Chipcard::PCSC::SCARD_PROTOCOL_T1
 sub new
 {
 	my $class = shift;
@@ -74,7 +74,7 @@ sub new
 	{
 		# Apply default values when required
 		$share_mode = $Chipcard::PCSC::SCARD_SHARE_EXCLUSIVE unless (defined($share_mode));
-		$prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0 unless (defined ($prefered_protocol));
+		$prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0 | $Chipcard::PCSC::SCARD_PROTOCOL_T1 unless (defined ($prefered_protocol));
 
 		($container->{hCard}, $container->{dwProtocol}) = Chipcard::PCSC::_Connect ($hContext->{hContext}, $reader_name, $share_mode, $prefered_protocol);
 		return (undef) unless (defined $container->{hCard});
@@ -90,8 +90,8 @@ sub new
 # Connect ($reader_name, $share_mode, $prefered_protocol)
 #
 # default values:
-# $share_mode = $Chipcard::PCSC::SCARD_SHARE_EXCLUSIVE
-# $prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0
+#  $share_mode = $Chipcard::PCSC::SCARD_SHARE_EXCLUSIVE
+#  $prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0 | $Chipcard::PCSC::SCARD_PROTOCOL_T1
 sub Connect
 {
 	my $self = shift;
@@ -114,7 +114,7 @@ sub Connect
 
 	# Apply default values when required
 	$share_mode = $Chipcard::PCSC::SCARD_SHARE_EXCLUSIVE unless (defined($share_mode));
-	$prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0 unless (defined ($prefered_protocol));
+	$prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0 | $Chipcard::PCSC::SCARD_PROTOCOL_T1 unless (defined ($prefered_protocol));
 
 	($self->{hCard}, $self->{dwProtocol}) = Chipcard::PCSC::_Connect ($self->{hContext}{hContext}, $reader_name, $share_mode, $prefered_protocol);
 
@@ -127,9 +127,9 @@ sub Connect
 # Reconnect ($share_mode, $prefered_protocol, $initialization)
 #
 # default values:
-# $share_mode = $Chipcard::PCSC::SCARD_SHARE_EXCLUSIVE
-# $prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0
-# $initialization = $Chipcard::PCSC::SCARD_LEAVE_CARD
+#  $share_mode = $Chipcard::PCSC::SCARD_SHARE_EXCLUSIVE
+#  $prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0 | $Chipcard::PCSC::SCARD_PROTOCOL_T1
+#  $initialization = $Chipcard::PCSC::SCARD_LEAVE_CARD
 sub Reconnect
 {
 	my $self = shift;
@@ -149,7 +149,7 @@ sub Reconnect
 
 	# Apply default values when required
 	$share_mode = $Chipcard::PCSC::SCARD_SHARE_EXCLUSIVE unless (defined($share_mode));
-	$prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0 unless (defined ($prefered_protocol));
+	$prefered_protocol = $Chipcard::PCSC::SCARD_PROTOCOL_T0 | $Chipcard::PCSC::SCARD_PROTOCOL_T1 unless (defined ($prefered_protocol));
 	$initialization = $Chipcard::PCSC::SCARD_LEAVE_CARD unless (defined($initialization));
 
 	$self->{dwProtocol} = Chipcard::PCSC::_Reconnect ($self->{hCard}, $share_mode, $prefered_protocol, $initialization);
@@ -215,6 +215,19 @@ sub Transmit
 	return (undef)
 } # Transmit
 
+sub Control
+{
+	my $self = shift;
+	confess ("wrong type") unless ref $self;
+
+	my $controlcode = shift;
+	my $send_data = shift;
+
+	confess ("not connected") unless defined $self->{hCard};
+
+	return Chipcard::PCSC::_Control ($self->{hCard}, $controlcode, $send_data);
+} # Control
+
 sub BeginTransaction
 {
 	my $self = shift;
@@ -359,7 +372,7 @@ $Chipcard::PCSC::Card::Error = "";
 # die "Error: $Chipcard::PCSC::Card::Error\n" unless defined $sw;
 #
 # Example:
-# ($sw, $res) = TransmitWithCheck("00 A4 01 00 02 01 00", "90 [1,0]0");
+# ($sw, $res) = TransmitWithCheck("00 A4 01 00 02 01 00", "90 [10]0");
 sub TransmitWithCheck
 {
 	my $self = shift;
@@ -371,6 +384,8 @@ sub TransmitWithCheck
 
 	my ($send, $recv, $data, $sw);
 
+	# add needed spaces
+	$command =~ s/(..)/$1 /g if ($command !~ m/ /);
 	print "=> $command\n" if (defined $debug);
 
 	# Convert $command in a reference to an array
